/****************************************************************************
**
** Definition of TQCommonStyle class
**
** Created : 980616
**
** Copyright (C) 2010 Timothy Pearson and (C) 1992-2008 Trolltech ASA.
**
** This file is part of the widgets module of the TQt GUI Toolkit.
**
** This file may be used under the terms of the GNU General
** Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the files LICENSE.GPL2
** and LICENSE.GPL3 included in the packaging of this file.
** Alternatively you may (at your option) use any later version
** of the GNU General Public License if such license has been
** publicly approved by Trolltech ASA (or its successors, if any)
** and the KDE Free TQt Foundation.
**
** Please review the following information to ensure GNU General
** Public Licensing requirements will be met:
** http://trolltech.com/products/qt/licenses/licensing/opensource/.
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
** or contact the sales department at sales@trolltech.com.
**
** This file may be used under the terms of the Q Public License as
** defined by Trolltech ASA and appearing in the file LICENSE.TQPL
** included in the packaging of this file.  Licensees holding valid TQt
** Commercial licenses may use this file in accordance with the TQt
** Commercial License Agreement provided with the Software.
**
** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
** herein.
**
**********************************************************************/

#ifndef TQCOMMONSTYLE_H
#define TQCOMMONSTYLE_H

#ifndef TQT_H
#include "tqstyle.h"
#endif // TQT_H

#ifndef TQT_NO_STYLE

class TQ_EXPORT TQCommonStyle: public TQStyle
{
    Q_OBJECT
    TQ_OBJECT

public:
    TQCommonStyle();
    ~TQCommonStyle();

	// Compatibility functions
	// For example, a TQt application may call tqdrawPrimitive with a ControlElement parameter
	// TQt "knows" that the application should have called tqdrawControl instead, and translates accordingly.
	inline void tqdrawPrimitive( TQ_ControlElement element, TQPainter *p, const TQRect &r, const TQColorGroup &cg, SFlags flags = Style_Default, const TQStyleOption &opt = TQStyleOption::Default ) const {
		TQ_UNUSED(r);
		TQ_UNUSED(cg);
		TQ_UNUSED(flags);
		drawControl((QStyle::ControlElement)(int)element, &opt, p, 0);	// [FIXME] What should widget really be?  I imagine 0 is *wrong*!!!
	}
	inline void tqdrawPrimitive( TQ_ComplexControl element, TQPainter *p, const TQRect &r, const TQColorGroup &cg, SFlags flags = Style_Default, const TQStyleOption &opt = TQStyleOption::Default ) const {
		TQ_UNUSED(r);
		TQ_UNUSED(cg);
		TQ_UNUSED(flags);
		drawComplexControl((QStyle::ComplexControl)(int)element, 0, p, 0);	// [FIXME] What should widget and complexcontrol really be?  I imagine 0 is *wrong*!!!
	}
	inline void tqdrawPrimitive( TQ_ControlElement element, TQPainter *p, const TQRect &r, const TQColorGroup &cg, int flags, const TQStyleOption &opt = TQStyleOption::Default ) const { tqdrawPrimitive( element, p, r, cg, (SFlags)flags, opt); }

    void tqdrawPrimitive( TQ_PrimitiveElement pe,
			TQPainter *p,
			const TQRect &r,
			const TQColorGroup &cg,
			SFlags flags = Style_Default,
			const TQStyleOption& = TQStyleOption::Default ) const;

    void tqdrawControl( TQ_ControlElement element,
		      TQPainter *p,
		      const TQWidget *widget,
		      const TQRect &r,
		      const TQColorGroup &cg,
		      SFlags how = Style_Default,
		      const TQStyleOption& = TQStyleOption::Default ) const;

    void tqdrawControlMask( TQ_ControlElement element,
			  TQPainter *p,
			  const TQWidget *widget,
			  const TQRect &r,
			  const TQStyleOption& = TQStyleOption::Default ) const;

    TQRect subRect( SubRect r, const TQWidget *widget ) const;

    void tqdrawComplexControl( TQ_ComplexControl control,
			     TQPainter *p,
			     const TQWidget *widget,
			     const TQRect &r,
			     const TQColorGroup &cg,
			     SFlags how = Style_Default,
#ifdef TQ_TQDOC
			     SCFlags sub = SC_All,
#else
			     SCFlags sub = (uint)SC_All,
#endif
			     SCFlags subActive = SC_None,
			     const TQStyleOption& = TQStyleOption::Default ) const;

    void tqdrawComplexControlMask( TQ_ComplexControl control,
				 TQPainter *p,
				 const TQWidget *widget,
				 const TQRect &r,
				 const TQStyleOption& = TQStyleOption::Default ) const;

    TQRect querySubControlMetrics( TQ_ComplexControl control,
				  const TQWidget *widget,
				  SubControl sc,
				  const TQStyleOption& = TQStyleOption::Default ) const;

    SubControl querySubControl( TQ_ComplexControl control,
				const TQWidget *widget,
				const TQPoint &pos,
				const TQStyleOption& = TQStyleOption::Default ) const;

    int tqpixelMetric( PixelMetric m, const TQWidget *widget = 0 ) const;

    TQSize tqsizeFromContents( ContentsType s,
			    const TQWidget *widget,
			    const TQSize &contentsSize,
			    const TQStyleOption& = TQStyleOption::Default ) const;

    int tqstyleHint(TQ_StyleHint sh, const TQWidget *, const TQStyleOption &, TQStyleHintReturn *) const;

    TQPixmap stylePixmap( StylePixmap sp,
			 const TQWidget *widget = 0,
			 const TQStyleOption& = TQStyleOption::Default ) const;


private:
    // Disabled copy constructor and operator=
#if defined(TQ_DISABLE_COPY)
    TQCommonStyle( const TQCommonStyle & );
    TQCommonStyle &operator=( const TQCommonStyle & );
#endif
};



#endif // TQT_NO_STYLE

#endif // TQCOMMONSTYLE_H
